//--------------------------------------------------------------------------------------------------
//
// Title       : Memory Mapped Serial Interface
// Design      : single
// Author      : Shen-Hui Lee
// Company     : UW
//
//-------------------------------------------------------------------------------------------------
//
// File        : Z:\pipelined\compile\MemoryMappedSerial.v
// Generated   : Tue May  2 11:33:43 2006
// From        : Z:\pipelined\src\Bootloader\MemoryMappedSerial.bde
// By          : Bde2Verilog ver. 2.01
//
//-------------------------------------------------------------------------------------------------
//
// Description : 
//
//-------------------------------------------------------------------------------------------------

`ifdef _VCP
`else
`define library
`endif


// ---------- Design Unit Header ---------- //
`timescale 1ps / 1ps

module MemoryMappedSerial_TF (RS232_RTS_IN,RS_232_RX_DATA,clk,reset,InAddress,InData,InWE,Size,RS_232_CTS_OUT,RS_232_TX_DATA,DataOut) ;

// ------------ Port declarations --------- //
input RS232_RTS_IN;
wire RS232_RTS_IN;
input RS_232_RX_DATA;
wire RS_232_RX_DATA;
input clk;
wire clk;
input reset;
wire reset;
input [5:0] InAddress;
wire [5:0] InAddress;
input [31:0] InData;
wire [31:0] InData;
input InWE;
wire InWE;
input [1:0]	Size;
wire [1:0] Size;
output RS_232_CTS_OUT;
wire RS_232_CTS_OUT;
output RS_232_TX_DATA;
wire RS_232_TX_DATA;
output [31:0] DataOut;
wire [31:0] DataOut;

// ----------- Signal declarations -------- //
wire NET1441;
wire NET2020;
wire ProcClock;
wire RxEmpty;
wire RxFIFOFull;
wire RxFIFORd;
wire RxFIFOWE;
wire sendFIFORead;
wire TxD_Busy;
wire TxD_Start;
wire TxFIFOFull;
wire TxFIFOWE;
wire [7:0] BUS1429;
wire [7:0] BUS1970;
wire [7:0] RxDataIn;
wire [7:0] RxFIFOOut;
wire [7:0] TxData;
wire [7:0] TxDIN;		 

parameter PROGRAM = "serial_tf.hex";
parameter PROGRAM_SIZE = 1024;

// -------- Component instantiations -------//
`library("InterfaceManager","lib378")
MemoryMappedSerialManager InterfaceManager
(
	.InAddress(InAddress),
	.InData(InData),
	.InWE(InWE),
	.Size(Size),
	.OutData(DataOut),
	.RecvFIFOEmpty(RxEmpty),
	.RecvFIFORead(RxFIFORd),
	.RecvFIFOTop(RxFIFOOut),
	.SendFIFOFull(TxFIFOFull),
	.SendFIFOWrite(TxFIFOWE),
	.clk(clk),
	.reset(reset),
	.sendFIFOData(TxDIN)
);

serial_tf
#(
.PRGM(PROGRAM),
.PRGM_SIZE(PROGRAM_SIZE))
TestFixture
( 
.CLK(clk),
.reset(reset),
.read_do(RxFIFORd),
.read_head(RxFIFOOut),
.read_empty(RxEmpty),
.write_do(TxFIFOWE),
.write_data(TxDIN),
.write_full(TxFIFOFull)
);

// ----------- Terminals assignment --------//
//	       ---- Input terminals ---         //

endmodule 

module serial_tf(CLK, reset, read_do, read_head, read_empty, write_do, write_data, write_full);
	parameter PRGM = "serial_tf.hex";
	parameter PRGM_SIZE = 1024;
	
	input CLK, reset;
	
	input read_do;
	output read_empty;
	output [7:0] read_head;
	
	input write_do;
	output write_full;
	input [7:0] write_data;
	
	reg [39:0] prgm [0:PRGM_SIZE];
	integer cmds_read, i; 
	integer current_byte;
	
	initial begin
		cmds_read = 0; 
		current_byte = 0;
		
		// clear out program memory
		for (i = 0; i < PRGM_SIZE; i = i + 1) begin
			prgm[i] = 40'b0;
		end
		
		// load program from PRGM
		$readmemh(PRGM, prgm);
	end
	
	always @(posedge CLK) begin
		if (reset) cmds_read = 0;
		if (read_do) begin 			 
			$display("serial_tf: advanced read pointer %d", $time);
			if(current_byte == 4) begin
				
				cmds_read = cmds_read + 1;
				current_byte = 0;
			end
			else begin
				current_byte = current_byte + 1;
			end
		end
		if (write_do) begin
			$display("serial_tf: Byte 0x%H written to the serial output @ %d", write_data,$time);
		end
	end
	
	assign read_head = (cmds_read >= PRGM_SIZE) ? 8'hx : (current_byte == 0) ? prgm[cmds_read][39:32] : (current_byte == 4) ? prgm[cmds_read][31:24] : (current_byte == 3) ? prgm[cmds_read][23:16] : (current_byte == 2) ? prgm[cmds_read][15:8] :  prgm[cmds_read][7:0];
	assign read_empty = (cmds_read >= PRGM_SIZE);
	
	assign write_full = 0;
endmodule